import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:phone_form_field/phone_form_field.dart';
import 'package:phone_form_field/src/country_selector.dart';
import 'package:phone_numbers_parser/phone_numbers_parser.dart';

import 'flag_dial_code_chip.dart';

enum SelectorDisplay {
  coversBody,
  // coversAll,
  coversLower,
}

/// Form Field for phone input
///
/// You can customize the phone number type that are valid with
/// the [PhoneNumberType] parameter.
///
/// To customize the look use [decoration], [inputTextStyle] and [cursorColor]
///
/// To customize how the country selection is displayed use the [selectorDisplay] param.
///
/// The rest of the parameters should be self explanatory or common to regular TextFormField.
class PhoneFormField extends FormField<PhoneNumber> {
  final bool autofocus;
  final bool showFlagInInput;

  /// input decoration applied to the input
  final InputDecoration decoration;
  final TextStyle inputTextStyle;
  final Color? cursorColor;
  final bool withHint;
  final ValueChanged<PhoneNumber?>? onChanged;

  /// the way the selector is shown
  final SelectorDisplay selectorDisplay;

  static String? Function(PhoneNumber?) _getDefaultValidator(
      PhoneNumberType? type) {
    return (PhoneNumber? phoneNumber) => phoneNumber == null ||
            phoneNumber.validate(type) ||
            phoneNumber.nsn == ''
        ? null
        : 'Invalid phone number';
  }

  PhoneFormField({
    // form field params
    Key? key,
    void Function(PhoneNumber)? onSaved,
    PhoneNumber? initialValue,
    bool enabled = true,
    AutovalidateMode autovalidateMode = AutovalidateMode.onUserInteraction,
    // our params
    this.onChanged,
    this.autofocus = false,
    this.showFlagInInput = true,
    this.decoration = const InputDecoration(border: UnderlineInputBorder()),
    this.inputTextStyle = const TextStyle(),
    this.cursorColor,
    this.withHint = true,
    this.selectorDisplay = SelectorDisplay.coversLower,
    PhoneNumberType? phoneNumberType,
  }) : super(
          key: key,
          initialValue: initialValue,
          onSaved: (p) => onSaved != null ? onSaved(p!) : (p) {},
          enabled: enabled,
          autovalidateMode: autovalidateMode,
          validator: _getDefaultValidator(phoneNumberType),
          builder: (field) {
            final state = field as _PhoneFormFieldState;
            return state.builder();
          },
        );

  @override
  _PhoneFormFieldState createState() => _PhoneFormFieldState();
}

class _PhoneFormFieldState extends FormFieldState<PhoneNumber> {
  final FocusNode _focusNode = FocusNode();
  final TextEditingController _controller = TextEditingController();

  _PhoneFormFieldState();

  get country => value?.country ?? Country.fromIsoCode('us');

  get isOutlineBorder => widget.decoration.border is OutlineInputBorder;

  @override
  PhoneFormField get widget => super.widget as PhoneFormField;

  @override
  void initState() {
    super.initState();
    _controller.text = widget.initialValue?.nsn ?? '';
    _controller.addListener(_onNationalNumberChanges);
  }

  @override
  void didChange(PhoneNumber? value) {
    super.didChange(value);
    if (widget.onChanged != null) {
      widget.onChanged!(value);
    }
  }

  @override
  void dispose() {
    _focusNode.dispose();
    _controller.dispose();
    super.dispose();
  }

  _onNationalNumberChanges() {
    final newPhoneNumber = PhoneNumber.fromIsoCode(
      country.isoCode,
      _controller.text,
    );
    didChange(newPhoneNumber);
  }

  _onCountrySelected(Country country) {
    PhoneNumber newPhoneNumber;
    if (value != null) {
      newPhoneNumber = value!.copyWithIsoCode(
        country.isoCode,
      );
    } else {
      newPhoneNumber = PhoneNumber.fromIsoCode(country.isoCode, '');
    }
    didChange(newPhoneNumber);
    _focusNode.requestFocus();
    Navigator.pop(context);
  }

  openCountrySelection() {
    final selector = CountrySelector(
      onCountrySelected: _onCountrySelected,
    );

    _focusNode.unfocus();

    if (widget.selectorDisplay == SelectorDisplay.coversBody) {
      showBottomSheet(
        context: context,
        builder: (_) => selector,
      );
    }
    // else if (widget.selectorDisplay == SelectorDisplay.coversAll) {
    //   showModalBottomSheet(
    //     context: context,
    //     builder: (_) => selector,
    //     // this param makes it fit the whole screen
    //     isScrollControlled: true,
    //   );
    // }
    else if (widget.selectorDisplay == SelectorDisplay.coversLower) {
      showModalBottomSheet(
        context: context,
        builder: (_) => selector,
      );
    }
  }

  Widget builder() {
    // the idea here is to have a TextField with a prefix where the prefix
    // is the flag + dial code which is the same height as text so it's well
    // aligned with the typed text. It also does not push labels etc
    // around and keep the same form factor as TextFormField.
    //
    // Then we stack an InkWell on top of that to add the clickable part
    return Stack(
      children: [
        _textField(),
        if (_focusNode.hasFocus) _inkWellOverlay(),
      ],
    );
  }

  Widget _textField() {
    return TextField(
      focusNode: _focusNode,
      controller: _controller,
      onSubmitted: (p) => widget.onSaved!(value),
      style: widget.inputTextStyle,
      autofocus: widget.autofocus,
      autofillHints: widget.enabled && widget.withHint
          ? [AutofillHints.telephoneNumberNational]
          : null,
      enabled: widget.enabled,
      textDirection: TextDirection.ltr,
      keyboardType: TextInputType.phone,
      cursorColor: widget.cursorColor,
      inputFormatters: <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
        FilteringTextInputFormatter.singleLineFormatter,
        // FilteringTextInputFormatter.allow(RegExp(r'^\d{0,30}$'))
      ],
      decoration: _getEffectiveDecoration(),
    );
  }

  Widget _inkWellOverlay() {
    return InkWell(
      onTap: openCountrySelection,
      // we make the country dial code
      // invisible but we still have to put it here
      // to have the correct width
      child: Opacity(
        opacity: 0,
        child: Padding(
          // outline border has padding on the left
          // so we need to make it a 12 bigger
          // and we add 16 horizontally to make it the whole height
          padding: isOutlineBorder
              ? const EdgeInsets.fromLTRB(12, 16, 0, 16)
              : const EdgeInsets.fromLTRB(0, 16, 0, 16),
          child: _getDialCode(),
        ),
      ),
    );
  }

  Widget _getDialCode() {
    return Padding(
      padding: const EdgeInsets.fromLTRB(8, 0, 8, 0),
      child: FlagDialCodeChip(
        country: country,
        showFlag: widget.showFlagInInput,
        textStyle: TextStyle(fontSize: 16),
        flagSize: 20,
      ),
    );
  }

  InputDecoration _getEffectiveDecoration() {
    return widget.decoration.copyWith(
      errorText: getErrorText(),
      prefix: _getDialCode(),
    );
  }

  // which error text to show
  String? getErrorText() {
    if (!hasError) return null;
    return PhoneFieldLocalization.of(context)?.translate(errorText!) ??
        errorText;
  }
}
